! =================================================================================================================================
! MODULE       : stomate_prescribe
!
! CONTACT      : orchidee-help _at_ ipsl.jussieu.fr
!
! LICENCE      : IPSL (2006)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF         Initialize and update density, height, basal area and crown area. For the resource
!                limited allocation and the allometric-based allocation
!!
!!\n DESCRIPTION: None
!!
!! RECENT CHANGE(S): None
!!
!! REFERENCE(S)	:
!!
!! SVN          :
!! $HeadURL: svn://forge.ipsl.jussieu.fr/orchidee/branches/ORCHIDEE-DOFOCO/ORCHIDEE/src_stomate/stomate_prescribe.f90 $
!! $Date: 2013-01-04 16:50:56 +0100 (Fri, 04 Jan 2013) $
!! $Revision: 1126 $
!! \n
!_ ================================================================================================================================

MODULE stomate_prescribe

  ! modules used:

  USE ioipsl_para
  USE stomate_data
  USE pft_parameters
  USE function_library,    ONLY: calculate_c0_alloc
  USE constantes

  IMPLICIT NONE

  ! private & public routines

  PRIVATE
  PUBLIC prescribe_diagnostic, prescribe_prognostic, prescribe_clear

    ! first call
  LOGICAL, SAVE                                      :: firstcall = .TRUE.
!$OMP THREADPRIVATE(firstcall)

CONTAINS


! =================================================================================================================================
!! SUBROUTINE   : prescribe_clear
!!
!>\BRIEF        : Set the firstcall flag back to .TRUE. to prepare for the next simulation.
!_=================================================================================================================================

  SUBROUTINE prescribe_clear
    firstcall=.TRUE.
  END SUBROUTINE prescribe_clear



!! ================================================================================================================================
!! SUBROUTINE   : prescribe_diagnostic
!!
!>\BRIEF         Works only with static vegetation and agricultural PFT. Initialize biomass,
!!               density, crown area in the first call and update them in the following calls.
!!
!! DESCRIPTION (functional, design, flags): \n
!! This module works only with static vegetation and agricultural PFT. In the first call, initialize 
!! density of individuals, biomass, crown area, and leaf age distribution to some reasonable value. 
!! In the following calls, these variables are updated.
!!
!! To fulfill these purposes, relationships from the pipe model are used:
!! \latexonly 
!!     \input{prescribe1.tex}
!!     \input{prescribe2.tex}
!! \endlatexonly
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLES(S): ::ind, ::cn_ind, ::leaf_frac
!!
!! REFERENCES   :
!! - Krinner, G., N. Viovy, et al. (2005). "A dynamic global vegetation model 
!!   for studies of the coupled atmosphere-biosphere system." Global 
!!   Biogeochemical Cycles 19: GB1015, doi:1010.1029/2003GB002199.
!! - Sitch, S., B. Smith, et al. (2003), Evaluation of ecosystem dynamics,
!!   plant geography and terrestrial carbon cycling in the LPJ dynamic 
!!   global vegetation model, Global Change Biology, 9, 161-185.
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================

 SUBROUTINE prescribe_diagnostic (npts, veget_max, dt, PFTpresent, everywhere, &
               when_growthinit, biomass, leaf_frac, ind, cn_ind, &
               co2_to_bm)


 !! 0. Parameters and variables declaration

   !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                           :: npts            !! Domain size (unitless)
    REAL(r_std), INTENT(in)                              :: dt              !! time step (dt_days)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)         :: veget_max       !! "maximal" coverage fraction of a PFT 
                                                                            !! (LAI -> infinity) on ground. 
                                                                            !! (unitless;0-1)
    !! 0.2 Output variables 

    !! 0.3 Modified variables

    LOGICAL, DIMENSION(npts,nvm), INTENT(inout)          :: PFTpresent      !! PFT present (0 or 1)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)      :: everywhere      !! is the PFT everywhere in the grid box or 
                                                                            !! very localized (after its introduction)
                                                                            !! (unitless;0-1)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)      :: when_growthinit !! how many days ago was the beginning of 
                                                                            !! the growing season (days)
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), &
                                           INTENT(inout) :: biomass         !! biomass @tex $(gC.m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nleafages), &
                                           INTENT(inout) :: leaf_frac       !! fraction of leaves in leaf age 
                                                                            !! class (unitless;0-1)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)      :: ind             !! density of individuals 
                                                                            !! @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)      :: cn_ind          !! crown area per individual 
                                                                            !! @tex $(m^{2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)      :: co2_to_bm       !! co2 taken up by carbohydrate 
                                                                            !! reserve at the beginning of the
                                                                            !! growing season @tex (gC.m^{-2}dt^{-1})$ @endtex 

    !! 0.4 Local variables

    REAL(r_std), DIMENSION(npts)                         :: dia             !! stem diameter (m)
    REAL(r_std), DIMENSION(npts,nelements)               :: woodmass        !! woodmass @tex $(gC.m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nelements)               :: woodmass_ind    !! woodmass of an individual 
                                                                            !! @tex $(gC.tree^{-1})$ @endtex    
    INTEGER(i_std)                                       :: i,j             !! index (unitless)

!_ ================================================================================================================================

 !! 1. Calculations at every call of the routine

    DO j = 2,nvm

      ! only when the DGVM is not activated or agricultural PFT.
      IF ( ( .NOT. control%ok_dgvm .AND. control%ok_constant_mortality ) .OR. ( .NOT. natural(j) ) ) THEN

        !! 1.1 Update crown area
        cn_ind(:,j) = zero

        IF ( is_tree(j) ) THEN

          !! 1.1.1 Trees
          dia(:) = zero
          DO i = 1, npts ! loop over grid points

            IF ( veget_max(i,j) .GT. zero ) THEN

              !! 1.1.1.1 calculate wood mass on an area basis, which include sapwood and heartwood aboveground and belowground.
              woodmass(i,icarbon) = (biomass(i,j,isapabove,icarbon) + biomass(i,j,isapbelow,icarbon) + &
                   biomass(i,j,iheartabove,icarbon) + biomass(i,j,iheartbelow,icarbon)) * veget_max(i,j)  

              IF ( woodmass(i,icarbon) .GT. min_stomate ) THEN

                !! 1.1.1.2 calculate critical individual density
                !?? the logic for 1.1.3 and 1.1.2 is strange, it should be the case that first to calculate critical woodmass 
                !??per individual, then calculate critical density.
                ! how to derive the following equation:
                ! first, TreeHeight=pipe_tune2 * Diameter^{pipe_tune3}
                ! we assume the tree is an ideal cylinder, so it volume is: 
                ! Volume = pi*(Dia/2)^2*Height = pi/4 * Dia * pipe_tune2*Dia^{pipe_tune3} 
                !        = pi/4 * pipe_tune2 * Dia^{2+pipe_tune3}
                ! last, the woodmass_per_individual = pipe_density * Volume = pipe_density*pi/4.*pipe_tune2 * Dia^{2+pipe_tune3}             
                ind(i,j) = woodmass(i,icarbon) / &
                           ( pipe_density(j)*pi/4.*pipe_tune2(j) * maxdia(j)**(2.+pipe_tune3(j)) )


                !! 1.1.1.3 individual biomass corresponding to this critical density of individuals
                woodmass_ind(i,icarbon) = woodmass(i,icarbon) / ind(i,j)

                !! 1.1.1.4 calculate stem diameter per individual tree
                dia(i) = ( woodmass_ind(i,icarbon) / ( pipe_density(j) * pi/4. * pipe_tune2(j) ) ) ** &
                         ( un / ( 2. + pipe_tune3(j) ) )

                !! 1.1.1.5 calculate provisional tree crown area for per individual tree
                !  equation: CrownArea=pipe_tune1 * Diameter^{1.6}
                cn_ind(i,j) = pipe_tune1(j) * MIN( maxdia(j), dia(i) ) ** pipe_tune_exp_coeff(j)

                !! 1.1.1.6 If total tree crown area for this tree PFT exceeds its veget_max, tree density is recalculated.
                IF ( cn_ind(i,j) * ind(i,j) .GT. 1.002* veget_max(i,j) ) THEN

                  ind(i,j) = veget_max(i,j) / cn_ind(i,j)

                ELSE

                   ind(i,j) = ( veget_max(i,j) / ( pipe_tune1(j) * (woodmass(i,icarbon)/(pipe_density(j)*pi/4.*pipe_tune2(j))) &
                        &     **(pipe_tune_exp_coeff(j)/(2.+pipe_tune3(j))) ) ) &
                        &     ** (1./(1.-(pipe_tune_exp_coeff(j)/(2.+pipe_tune3(j)))))
                   woodmass_ind(i,icarbon) = woodmass(i,icarbon) / ind(i,j)
                   dia(i) = ( woodmass_ind(i,icarbon) / ( pipe_density(j) * pi/4. * pipe_tune2(j) ) ) ** &
                        &     ( un / ( 2. + pipe_tune3(j) ) )
                   cn_ind(i,j) = pipe_tune1(j) * MIN( maxdia(j), dia(i) ) ** pipe_tune_exp_coeff(j)

                ENDIF

              ELSE ! woodmass=0  => impose some value

                dia(:) = maxdia(j)
                cn_ind(i,j) = pipe_tune1(j) * MIN( maxdia(j), dia(i) ) ** pipe_tune_exp_coeff(j)

              ENDIF ! IF ( woodmass .GT. min_stomate )

            ENDIF    ! veget_max .GT. 0.

          ENDDO      ! loop over grid points

        !! 1.2 grasses: crown area always set to 1m**2
        ELSE

          WHERE ( veget_max(:,j) .GT. zero )

            cn_ind(:,j) = un

          ENDWHERE

        ENDIF   !IF ( is_tree(j) )
     
        !! 1.3 density of individuals
        WHERE ( veget_max(:,j) .GT. zero )

          ind(:,j) = veget_max(:,j) / cn_ind(:,j)  

        ELSEWHERE

          ind(:,j) = zero

        ENDWHERE

      ENDIF     ! IF ( ( .NOT. control%ok_dgvm .AND. control%ok_constant_mortality ) .OR. ( .NOT. natural(j) ) )

    ENDDO       ! loop over PFTs


 !! 2. If it's the first call for this module, 

    IF (( firstcall ) .AND. (TRIM(stom_restname_in) == 'NONE')) THEN

      ! impose some biomass if zero and PFT prescribed
      WRITE(numout,*) 'prescribe:'
      WRITE(numout,*) '   > Imposing initial biomass for prescribed trees, '// &
                      'initial reserve mass for prescribed grasses.'
      WRITE(numout,*) '   > Declaring prescribed PFTs present.'

      DO j = 2,nvm ! loop over PFTs

        DO i = 1, npts ! loop over grid points

          ! is vegetation static or PFT agricultural?
          ! Static vegetation or agricultural PFT
          IF ( ( .NOT. control%ok_dgvm ) .OR. &
               ( ( .NOT. natural(j) ) .AND. ( veget_max(i,j) .GT. min_stomate ) ) ) THEN

            
            !! 2.1 if tree biomass is extremely small, prescribe the biomass by assuming they have sapling biomass, 
            !! which is a constant in the model then set all the leaf age as 1.
            !! if tree PFT and biomass too small, prescribe the biomass to a value.
            IF ( is_tree(j) .AND. ( veget_max(i,j) .GT. min_stomate ) .AND. &
                 ( SUM( biomass(i,j,:,icarbon) ) .LE. min_stomate ) ) THEN

               !!? here the code is redundant, as "veget_max(i,j) .GT. min_stomate" is already met in the above if 
               !!? condition.
               IF (veget_max(i,j) .GT. min_stomate) THEN

                  biomass(i,j,:,:) = (bm_sapl_rescale * bm_sapl_old(j,:,:) * ind(i,j)) / veget_max(i,j)

               ELSE

                  biomass(i,j,:,:) = zero

               ENDIF

              ! set leaf age classes
              leaf_frac(i,j,:) = zero
              leaf_frac(i,j,1) = un

              ! set time since last beginning of growing season
              when_growthinit(i,j) = large_value

              ! seasonal trees: no leaves at beginning
              IF ( pheno_model(j) .NE. 'none' ) THEN

                biomass(i,j,ileaf,icarbon) = zero
                leaf_frac(i,j,1) = zero

              ENDIF

              co2_to_bm(i,j) = co2_to_bm(i,j) + ( SUM(biomass(i,j,:,icarbon))  / dt )

            ENDIF

            !! 2.2 for grasses, set only the carbon reserve pool to "sapling" carbon reserve pool.
            !!     and set all leaf age to 1.
            IF ( ( .NOT. is_tree(j) ) .AND. ( veget_max(i,j) .GT. min_stomate ) .AND. &
                 ( SUM( biomass(i,j,:,icarbon) ) .LE. min_stomate ) ) THEN

              !+++++ CHECK MJM ++++++
               ! bm_sapl will also be a function of the circumference class...just using the first class right now
               biomass(i,j,icarbres,:) = bm_sapl_old(j,icarbres,:) * ind(i,j) / veget_max(i,j)

              ! set leaf age classes
              leaf_frac(i,j,:) = zero
              leaf_frac(i,j,1) = un

              ! set time since last beginning of growing season
              when_growthinit(i,j) = large_value

              co2_to_bm(i,j) = co2_to_bm(i,j) + ( biomass(i,j,icarbres,icarbon)  / dt )
            
            ENDIF

            !! 2.3 declare all PFTs with positive veget_max as present everywhere in that grid box
            IF ( veget_max(i,j) .GT. min_stomate ) THEN

              PFTpresent(i,j) = .TRUE.
              everywhere(i,j) = un

            ENDIF

          ENDIF   ! not control%ok_dgvm  or agricultural

        ENDDO ! loop over grid points

      ENDDO ! loop over PFTs

      firstcall = .FALSE.

    ENDIF

  END SUBROUTINE prescribe_diagnostic


!! ================================================================================================================================
!! SUBROUTINE   : prescribe_prognostic
!!
!>\BRIEF         Works only with static vegetation and agricultural PFT. Initialize biomass,
!!               density, presence in the first call and update them in the following.
!!
!! DESCRIPTION (functional, design, flags): \n
!! This module works only with static vegetation and agricultural PFT.
!! In the first call, initialize density of individuals, biomass,
!! and leaf age distribution to some reasonable value. In the following calls,
!! these variables are updated.
!!
!! To fulfill these purposes, pipe model are used:
!! \latexonly 
!!     \input{prescribe1.tex}
!!     \input{prescribe2.tex}
!! \endlatexonly
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLES(S): ::ind, ::cn_ind, ::leaf_frac
!!
!! REFERENCES   :
!! - Krinner, G., N. Viovy, et al. (2005). "A dynamic global vegetation model 
!!   for studies of the coupled atmosphere-biosphere system." Global 
!!   Biogeochemical Cycles 19: GB1015, doi:1010.1029/2003GB002199.
!! - Sitch, S., B. Smith, et al. (2003), Evaluation of ecosystem dynamics,
!!   plant geography and terrestrial carbon cycling in the LPJ dynamic 
!!   global vegetation model, Global Change Biology, 9, 161-185.
!! - McDowell, N., Barnard, H., Bond, B.J., Hinckley, T., Hubbard, R.M., Ishii, 
!!   H., Köstner, B., Magnani, F. Marshall, J.D., Meinzer, F.C., Phillips, N., 
!!   Ryan, M.G., Whitehead D. 2002. The relationship between tree height and leaf 
!!   area: sapwood area ratio. Oecologia, 132:12–20.
!! - Novick, K., Oren, R., Stoy, P., Juang, F.-Y., Siqueira, M., Katul, G. 2009. 
!!   The relationship between reference canopy conductance and simplified hydraulic 
!!   architecture. Advances in water resources 32, 809-819. 
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================

 SUBROUTINE prescribe_prognostic (npts, veget_max, veget, dt, PFTpresent, &
               everywhere, when_growthinit, biomass, leaf_frac, &
               ind, circ_class_n, circ_class_biomass, co2_to_bm, &
               forest_managed, KF, &
               senescence, age, npp_longterm, lm_lastyearmax, &
               tau_eff_leaf, tau_eff_sap, tau_eff_root, k_latosa_adapt)

!! 0. Parameters and variables declaration

   !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                        :: npts              !! Domain size (unitless)
    INTEGER(i_std), DIMENSION (:,:), INTENT(in)       :: forest_managed    !! forest management flag
    REAL(r_std), INTENT(in)                           :: dt                !! time step (dt_days)   
    REAL(r_std), DIMENSION(:,:), INTENT(in)           :: veget_max         !! "maximal" coverage fraction of a PFT 
                                                                           !! (LAI -> infinity) on ground. May sum to
                                                                           !! less than unity if the pixel has
                                                                           !! nobio area. (unitless; 0-1)
    REAL(r_std), DIMENSION(:,:), INTENT(in)           :: veget             !! Fraction of vegetation type including 
                                                                           !! non-biological fraction (unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(in)           :: tau_eff_root      !! Effective root turnover time that accounts
                                                                           !! waterstress (days)
    REAL(r_std), DIMENSION(:,:), INTENT(in)           :: tau_eff_sap       !! Effective sapwood turnover time that accounts
                                                                           !! waterstress (days)
    REAL(r_std), DIMENSION(:,:), INTENT(in)           :: tau_eff_leaf      !! Effective leaf turnover time that accounts
                                                                           !! waterstress (days)

    !! 0.2 Output variables
 
   
    !! 0.3 Modified variables

    LOGICAL, DIMENSION(:,:), INTENT(inout)            :: PFTpresent         !! PFT present (0 or 1)
    LOGICAL, DIMENSION(:,:), INTENT(inout)            :: senescence         !! Flag for setting senescence stage (only 
                                                                            !! for deciduous trees)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)        :: everywhere         !! is the PFT everywhere in the grid box or 
                                                                            !! very localized (after its introduction) (?)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)        :: when_growthinit    !! how many days ago was the beginning of 
                                                                            !! the growing season (days)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)        :: ind                !! Density of individuals at the stand level 
                                                                            !! @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)        :: npp_longterm       !! "long term" net primary productivity
                                                                            !! @tex ($gC m^{-2} year^{-1}$) @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)        :: age                !! mean age (years)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)        :: lm_lastyearmax     !! last year's maximum leaf mass for each PFT 
                                                                            !! @tex ($gC m^{-2}$) @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)      :: circ_class_n       !! Number of individuals in each circ class
                                                                            !! @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)    :: biomass            !! Stand level biomass 
                                                                            !! tex $(gC.m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:,:,:,:), INTENT(inout)  :: circ_class_biomass !! Biomass components of the model tree  
                                                                            !! within a circumference class
                                                                            !! class @tex $(g C ind^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)        :: co2_to_bm          !! CO2 taken from the atmosphere to get C
                                                                            !! to create the seedlings 
                                                                            !!  @tex (gC.m^{-2}dt^{-1})$ @endtex  
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)      :: leaf_frac          !! fraction of leaves in leaf age 
                                                                            !! class (unitless;0-1)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)        :: KF                 !! Scaling factor to convert sapwood mass
                                                                            !! into leaf mass (m) - this variable is 
                                                                            !! passed to other routines
    REAL(r_std), DIMENSION(:,:), INTENT(inout)        :: k_latosa_adapt     !! Leaf to sapwood area adapted for long 
                                                                            !! term water stress (m)
  

    !! 0.4 Local variables

    REAL(r_std), DIMENSION(npts,nvm)                   :: c0_alloc           !! Root to sapwood tradeoff parameter
    INTEGER(i_std)                                     :: ipts, ivm, ipar    !! index (unitless)
    INTEGER(i_std)                                     :: icir, iele, imbc   !! index (unitless)
    INTEGER(i_std)                                     :: deb,fin, imaxt     !! index (unitless)
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements)  :: sync_biomass       !! Temporary stand level biomass 
                                                                             !! @tex $(gC.m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm)                   :: sync_ind           !! Temporary density of individuals at the 
                                                                             !! stand level @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm)                   :: k_latosa           !! Height dependent base value to calculate 
                                                                             !! KF (-) 
    REAL(r_std), DIMENSION(nvm,ncirc,nparts,nelements) :: bm_sapl            !! Sapling biomass for the functional 
                                                                             !! allocation with a dimension for the 
                                                                             !! circumference classes
                                                                             !! @tex $(gC.ind^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm)                   :: LF                 !! Scaling factor to convert sapwood mass
                                                                             !! into root mass (unitless)
    REAL(r_std), DIMENSION(npts,nvm)                   :: lstress_fac        !! Light stress factor, based on total
                                                                             !! transmitted light (unitless, 0-1)
    REAL(r_std), DIMENSION(npts,nvm,ncirc)             :: circ_class         !! circumference of individual trees
    REAL(r_std)                                        :: lambda             !! lambda of the truncated exponential 
                                                                             !! distribution of initial diameters 
                                                                             !! (1/cm**2). Parameterized based on Dhote 
                                                                             !! (2003): lambda = sqrt(2)/Dg and  
                                                                             !! Dginit = 1 cm. 
    REAL(r_std)                                        :: p_max              !! Cumulated probability level at which the 
                                                                             !! exponential distribution truncated 
                                                                             !! (dimensionless, 0-1) 
    REAL(r_std),DIMENSION(ncirc)                       :: circ_ij0           !! Circumference of all individual trees  
    REAL(r_std),DIMENSION(ncirc)                       :: height_ij0         !! Height of each Circumference of all 
                                                                             !! individual trees  
                                                                             !! for 1 ha at gridpoint i and for PFT j  
                                                                             !! (m). As above, 0 is before thinning 
    REAL(r_std), DIMENSION(nparts)                     :: bm_init            !! Biomass needed to initiate the next 
                                                                             !! planting @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(ncirc)                      :: nb_trees_i         !! Number of trees in each twentith 
                                                                             !! circumference quantile of the 
                                                                             !! distribution (ind) 
    REAL(r_std)                                        :: excedent           !! Number of trees after truncation to be 
                                                                             !! reallocated to smaller quantiles of the 
                                                                             !! distribution (ind) 
    REAL(r_std)                                        :: max_circ_init      !! Maximum initial circumferences of the 
                                                                             !! truncated exponential distribution (cm) 
    REAL(r_std), DIMENSION(ncirc)                      :: ave_tree_height    !! The height of the ideal tree in each
                                                                             !! circumference class of the 
                                                                             !! distribution (ind)...not saved since
                                                                             !! it should only be used for prescribing
    REAL(r_std), DIMENSION(npts,nvm,nmbcomp,nelements) :: check_intern       !! Contains the components of the internal
                                                                             !! mass balance chech for this routine
                                                                             !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)         :: closure_intern     !! Check closure of internal mass balance
                                                                             !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)         :: pool_start         !! Start and end pool of this routine 
                                                                             !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)         :: pool_end           !! Start and end pool of this routine 
                                                                             !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std)                                        :: delta_KF           !! Difference between new and old estimate 
                                                                             !! of KF while iterating
    REAL(r_std)                                        :: min_height_init           
    REAL(r_std)                                        :: max_height_init           
    REAL(r_std)                                        :: sapwood_density
    REAL(r_std)                                        :: dia_init           
    REAL(r_std)                                        :: wood_init                    
    REAL(r_std), DIMENSION(ncirc)                      :: height_dist  
    INTEGER(i_std)                                     :: iloop

    !++++ temp ++++
!!$    REAL(r_std), DIMENSION(npts,nvm,ncirc)             :: height,dia,cn       !! Tree height calculated from allometric 
!!$                                                                              !! relationships (m) 
        
!_ ================================================================================================================================

 !! 1. Initialize biomass at first call

    !! 1.1.1 Initialize check for mass balance closure
    !  The mass balance is calculated at the end of this routine
    !  in section 4
    !  Initial biomass pool
    pool_start(:,:,:) = zero
    DO ipar = 1,nparts
       DO iele = 1,nelements
          pool_start(:,:,iele) = pool_start(:,:,iele) + &
               (biomass(:,:,ipar,iele) * veget_max(:,:))
       ENDDO
    ENDDO

   ! co2_to_bm is has as intent inout, the variable accumulates
   ! carbon over the course of a day. Use the difference between
   ! start and the end of this routine
   check_intern(:,:,iatm2land,icarbon) = - un * co2_to_bm(:,:) * veget_max(:,:) * dt

   ! Prescribe was taken out of the first call loop. Because of the firstcall here, 
   ! nothing was done when the vegetation was killed in lpj_gap and the vegetation
   ! thus stayed dead forever.  We want it to regrow.  So instead, let's loop over 
   ! every point and PFT.  If there is supposed to be vegetation here (according to 
   ! veget_max) but there isn't (according to ind), then we grow it.
   DO ivm = 2,nvm ! Loop over PFTs

      DO ipts = 1, npts ! Loop over pixels

         ! We are supposed to have vegetation, but we don't have any: prescribe some.
         IF((veget_max(ipts,ivm) .GT. min_stomate) .AND. (ind(ipts,ivm) .LE. min_stomate))THEN

            ! Initilaize tree level variables
            circ_class_n(ipts,ivm,:) = zero
            biomass(ipts,ivm,:,:) = zero
            circ_class_biomass(ipts,ivm,:,:,:) = zero

            ! Write output
            IF (ld_presc) THEN
               WRITE(numout,*) 'Prescribe (stomate_prescribe.f90):'
               WRITE(numout,*) '   > Imposing initial biomass for prescribed trees, '// &
                    'initial reserve mass for prescribed grasses.'
               WRITE(numout,*) '   > Declaring prescribed PFTs present.'
               WRITE(numout,*) '   > Grid point: ',ipts,' PFT Type: ',ivm
            ENDIF

            !! 2. Calculate the vegetation characteristics of a newly established vegetation

            !! 2.1 Stress factors

            ! Note that light and water stress have an opposite effect on KF. Waterstress 
            ! will decrease KF because more C should be allocated to the roots. Light 
            ! stress will increase KF because more C should be allocated to the leaves.

            ! We will need the c0_alloc, it only depends on PFT and the effective 
            ! longiveties
            c0_alloc(ipts,ivm) = calculate_c0_alloc(ipts, ivm, tau_eff_root(ipts,ivm), &
                 tau_eff_sap(ipts,ivm))

            ! Lightstress varies from 0 to 1 and is calculated from the canopy structure (veget)
            ! Given that there is no vegetation at this point, the lightstress cannot be calculated 
            ! and is set to 1. However as soon as we grow a canopy it will experience light stress
            ! and so KF should be adjusted. If this is not done, we can't prescribe tall vegetation
            ! which is a useful feature to speed up optimisation and testing. We will have iterate
            ! over light stress and KF to prescribe vegetation that is in balance with its 
            ! light environment. 
            lstress_fac(ipts,ivm) = un

            ! Initial vegetation has to be prescribed only when the vegetation is static
            IF ( ( .NOT. control%ok_dgvm ) .AND. &
                 ( veget_max(ipts,ivm) .GT. min_stomate ) )  THEN

               !! 2.2 Initialize woody PFT's 
               !      Use veget_max to check whether the PFT is present. If the PFT is present but it has no
               !      biomass, prescribe its biomass (gC m-{2}).
               IF ( is_tree(ivm) .AND. &
                    ( veget_max(ipts,ivm) .GT. min_stomate ) .AND. &
                    ( SUM( biomass(ipts,ivm,:,icarbon) ) .LE. min_stomate ) ) THEN

                  ! PFT is present so it needs to be initialized 
                  IF (veget_max(ipts,ivm) .GT. min_stomate) THEN

                     ! The forest stand starts with the number of individuals as prescribed in constantes_mtc.f90
                     ! This number is defined per hectare whereas these calculations are per m2
                     ind(ipts,ivm) = nmaxtrees(ivm) * ha_to_m2   

                     !! 2.3 Initialize height distribution
                     !  Initializing height distribution...I've taken this from the circumference initilization of Valentin
                     !  we want ncirc bins between height_init_min and height_init_max
                     lambda = SQRT(2.0_r_std)/pi 
                     min_height_init = height_init_min(ivm)
                     max_height_init = height_init_max(ivm)
                     p_max = 100./nmaxtrees(ivm)
                     height_dist(:) = zero

                     !! 2.4 Calculate height distribution

                     !! 2.4.1 Height distribution of a high stand
                     !  Deterministic initial distribution following a truncated exponential law 
                     !  if they are coppices, we handle this differently

                     !++++ CHECK+++++
                     ! I am treating SRC the same way as everything else for now.
                     !IF (forest_managed(ipts,ivm) .NE. ifm_src) THEN
                     nb_trees_i(:) = 0

                     ! The distribution is truncated when P(X>max_circ_init)<p_max
                     ! The circumference distribution is divided into ncirc quantiles of equal height range. 
                     ! A integer number of trees is 
                     ! allocated to each quantile according to the truncated exponential distribution.
                     DO icir = 1,ncirc ! loop over circumference quantiles

                        nb_trees_i(icir) = NINT(nmaxtrees(ivm)*&
                             (EXP(-lambda*max_height_init*(icir-1)/ncirc)-EXP(-lambda*max_height_init*icir/ncirc)))
                        ! this array is going to be used to create the sapling for this class so that we can create 
                        ! the total amount of biomass
                        ! in this class...after this we will always calculate the height/circ/diam/whatever 
                        ! from the biomass in the class
                        ave_tree_height(icir)=min_height_init+(REAL(icir,r_std)-0.5_r_std)*&
                             (max_height_init-min_height_init)/REAL(ncirc,r_std)

                        ! I am changing this so that our lowest class is min_height_init and our biggest class
                        ! is max_height_init.  With this change, we can use the same code the redistribute
                        ! the trees if the biomass in one of our height classes is equal to zero later on due to mortality or
                        ! thinning.
                        !+++ This caused some strange behavior, so change it back for now.
                        !                        ave_tree_height(icir)=min_height_init+REAL(icir-1,r_std)/REAL(ncirc-1,r_std)*&
                        !                             (max_height_init-min_height_init)

                     ENDDO

                     excedent = REAL(nmaxtrees(ivm) - SUM(nb_trees_i))
                     nb_trees_i = nb_trees_i + FLOOR(excedent/ncirc)
                     nb_trees_i(1) = nb_trees_i(1) + NINT(excedent)-ncirc*FLOOR(excedent/ncirc)

                     IF (ld_presc) THEN
                        WRITE(numout,*)"min initial height ",min_height_init,'max initial height ',max_height_init
                        WRITE(numout,*)'nb_trees_i',nb_trees_i
                     ENDIF

!                     ENDIF ! check FM type

                     ! we need to store the number of trees that we have per square meter for each circumference class
                     circ_class_n(ipts,ivm,:) = REAL(nb_trees_i(:),r_std) * ha_to_m2 

                     !! 2.4.2 Circumference distribution of a coppice
                     ! I'm not see why this should be any different than a standard prescribe.
                     ! Ideally, the plantings should be stems 20-25 cm long stuck into the soil,
                     ! but the allocation scheme will have the same issues with this that it does
                     ! with planting a small sapling, i.e. it won't like it.  The stems do not
                     ! follow standard allocation rules.  Then again, neither do coppices.

!!$                     IF (forest_managed(ipts,ivm) == ifm_src) THEN
!!$                        WRITE(numout,*) '!********* Problem with SRC ******'
!!$                        STOP
!!$                     ENDIF

                     !! 2.5 Allocation factors
                     !  Sapwood to root ratio
                     !  Following Magnani et al. 2000 "In order to decreases hydraulic resistance,
                     !  the investment of carbon in fine roots or sapwood yields to the plant very
                     !  different returns, both because of different hydraulic conductivities and 
                     !  because of the strong impact of plant height on shoot resistance. On the 
                     !  other hand, fine roots and sapwood have markedly different longevities and 
                     !  the cost of production, discounted for turnover, will differ accordingly.
                     !  Optimal growth under hydraulic constraints requires that the ratio of marginal 
                     !  hydraulic returns to marginal annual cost for carbon investment in either roots 
                     !  or sapwood be the same (Bloom, Chapin & Mooney 1985; Case & Fair 1989). This
                     !  is formalized in equation (13) and further derived to obtain equation (17) 
                     !  in Magnani et al 2000. The latter is implemented here.
                     !  Pipe_density is given in gC/m-3, convert to kg/m-3. And apply equation (17) 
                     !  in Magnani et al 2000. Note that c0_alloc was calculated at the start of this 
                     !  routine. The calculation itself is done in function_library
                     
                     ! Calculate leaf area to sapwood area
                     ! To be consistent with the hydraulic limitations and pipe theory,
                     ! k_latosa is calculated from equation (18) in Magnani et al.
                     ! To do so, total hydraulic resistance and tree height need to known. This
                     ! poses a problem as the resistance depends on the leaf area and the leaf 
                     ! area on the resistance. There is no independent equation and equations 12
                     ! and 18 depend on each other and substitution would be circular. Hence 
                     ! prescribed k_latosa values were obtained from observational records 
                     ! and are given in mtc_parameters.f90. 

                     ! The relationship between height and k_latosa as reported in McDowell 
                     ! et al 2002 and Novick et al 2009 is implemented to adjust k_latosa for 
                     ! the height of the stand. This did NOT result in a realistic model behavior 
                     !!$ k_latosa(ipts,ivm) = wstress_fac(ipts,ivm) * &
                     !!$    (k_latosa_max(ivm) - (latosa_height(ivm) * &
                     !!$    (SUM( nb_trees_i(:) * ave_tree_height(:) ) / SUM( nb_trees_i(:) ))))

                     ! Alternatively, k_latosa is also reported to be a function of diameter 
                     ! (i.e. stand thinning, Simonin et al 2006, Tree Physiology, 26:493-503).
                     ! Here the relationship with thinning was interpreted as a realtionship with
                     ! light stress. Note that light stress cannot be calculated at this time in
                     ! the model because there is no canopy (that's why we are in prescribe!) and
                     ! so there is no lightstress. lstress was therefore set to one (see above).
                     ! We prefered this redundancy in the code because it makes it clear that
                     ! k_latosa is calculated in the same way in prescribe.f90 and growth_fun_all.f90
                     ! +++CHECK+++
                     ! How do we want to deal with waterstress
!!$                     k_latosa(ipts,ivm) = k_latosa_min(ivm) + &
!!$                          (wstress_fac(ipts,ivm) * lstress_fac(ipts,ivm) * &
!!$                          (k_latosa_max(ivm)-k_latosa_min(ivm)))
!!$                     k_latosa(ipts,ivm) = wstress_fac(ipts,ivm) * (k_latosa_min(ivm) + &
!!$                          (lstress_fac(ipts,ivm) * &
!!$                          (k_latosa_max(ivm)-k_latosa_min(ivm))))
                     k_latosa(ipts,ivm) = (k_latosa_adapt(ipts,ivm) + &
                            (lstress_fac(ipts,ivm) * &
                          (k_latosa_max(ivm)-k_latosa_min(ivm))))
                     ! ++++++++++++
                     

                     ! Also k_latosa has been reported to be a function of CO2 concentration 
                     ! (Atwell et al. 2003, Tree Physiology, 23:13-21 and Pakati et al. 2000, 
                     ! Global Change Biology, 6:889-897). This effect is not accounted for in 
                     ! the current code

                     ! Calculate conversion coefficient for sapwood area to leaf area 
                     ! (1) The scaling parameter between leaf and sapwood mass is derived from
                     ! LA_ind = k_latosa * SA_ind, where LA_ind = leaf area of an individual, SA_ind is the 
                     ! sapwood area of an individual and k_latosa a pipe-model parameter
                     ! (2) LA_ind = Cl * sla
                     ! (3) Cs = SA_ind * height * wooddensity * tree_ff
                     ! Substitute (2) and (3) in (1)
                     ! Cl = Cs * k1 / (wooddensity * sla * tree_ff * height)
                     ! Cl = Cs*KF/height, where KF is in (m)
                     ! KF is passed to the allocation routine and it is saved in the restart file.
                     KF(ipts,ivm) = k_latosa(ipts,ivm) / ( sla(ivm) * pipe_density(ivm) * tree_ff(ivm))
                     
                     ! Initialize delta_KF to get the DO WHILE started
                     delta_KF = un

                     iloop=0
                     DO WHILE (delta_KF .GT. max_delta_KF)

                        !  If there is a WHILE loop, there always needs to be a check on the number
                        ! of loops to make sure we don't get stuck in an infinite loop.  This
                        ! number is completely arbitrary.
                        iloop=iloop+1
                        IF(iloop > 1000)THEN
                           WRITE(numout,*) 'Taking too long to converge the delta_KF loop in prescribe!'
                           WRITE(numout,*) 'iloop,delta_KF,max_delta_KF,ivm,ipts: ',&
                                iloop,delta_KF,max_delta_KF,ivm,ipts
                           STOP
                        ENDIF

                        !! 2.6 Create saplings for each height class
                        !  Now we create the saplings for each class, based on the height
                        DO icir = 1,ncirc
                           
                           ! The assumption we make is that we plant trees of 2 to 3 years old rather than 
                           ! growing trees from seeds. The allometric relationship between height and 
                           ! diameter is derived from mature tree and likely unrealistic for saplings. 
                           ! The height of the saplings is prescribed and determines the reserves which are 
                           ! especially important for deciduous species which need to survive on their 
                           ! reserves for the first year (new phenology scheme requires annual mean values
                           ! to get started) 
                           dia_init = ( ave_tree_height(icir) / pipe_tune2(ivm) ) ** ( 1. / pipe_tune3(ivm) )
                           wood_init = ( ave_tree_height(icir) * pi / 4. * (dia_init) ** 2. ) * &
                                pipe_density(ivm) * tree_ff(ivm)
                           
                           ! The woody biomass is contained in four components. Thus, wood_init = isapabove + 
                           ! isapbelow + iheartabove + iheartbelow. Given that isapbelow = isapbelow and 
                           ! iheartabove = iheartbelow =  bm_sapl_heartabove*isapabove. If bm_sapl_heartbelow = 
                           ! bm_sapl_heartabove = 0.2, then isapabove = wood_init/2.4
                           bm_sapl(ivm,icir,isapabove,icarbon) = wood_init / (2. + bm_sapl_heartabove + bm_sapl_heartbelow) 
                           bm_sapl(ivm,icir,isapbelow,icarbon) = bm_sapl(ivm,icir,isapabove,icarbon)
                           bm_sapl(ivm,icir,iheartabove,icarbon) =  bm_sapl_heartabove * bm_sapl(ivm,icir,isapabove,icarbon)
                           bm_sapl(ivm,icir,iheartbelow,icarbon) =  bm_sapl_heartbelow * bm_sapl(ivm,icir,isapbelow,icarbon)          
                           
                           ! Use the allometric relationships to calculate initial leaf and root mass 
                           bm_sapl(ivm,icir,ileaf,icarbon) = ( bm_sapl(ivm,icir,isapabove,icarbon) + &
                                bm_sapl(ivm,icir,isapbelow,icarbon) ) * KF(ipts,ivm) /  ave_tree_height(icir) 
                           !+++CHECK+++
                           !How do we want to deal with water stress? wstress is accounted for through c0
!!$                           bm_sapl(ivm,icir,iroot,icarbon) = bm_sapl(ivm,icir,ileaf,icarbon) / ( KF(ipts,ivm) * c0_alloc(ivm) )
                           bm_sapl(ivm,icir,iroot,icarbon) = bm_sapl(ivm,icir,ileaf,icarbon) / &
                                ( KF(ipts,ivm) * c0_alloc(ipts,ivm) )
                           !+++++++++++
                           
                           ! Write initial values
                           IF (ld_presc) THEN 
                              WRITE(numout,*) ' Circumference class: ',icir,' PFT type: ',ivm
                              WRITE(numout,*) '       root to sapwood tradeoff p :', c0_alloc(ipts,ivm)
                              WRITE(numout,*) 'height_init, dia_init, wood_init, ', &
                                   ave_tree_height(icir) , dia_init, wood_init
                              WRITE(numout,*) 'pipe_density, ',pipe_density(ivm)
                           ENDIF
                           
                           !++++++ CHECK ++++++
                           ! The carbohydrate reserves do not seem to be set before this line.  This
                           ! is a problem since it then uses an unitililized value.  Therefore, I
                           ! will initilize it.
                           ! Should this really be zero? If so the code below is nonesensical and icarbres
                           ! could be omitted. nonesensical code = 2 * (bm_sapl(ivm,icir,icarbres,icarbon) + &
                           !        bm_sapl(ivm,icir,ileaf,icarbon) + bm_sapl(ivm,icir,iroot,icarbon))
                           bm_sapl(ivm,icir,icarbres,icarbon)=zero
                           !++++++++++++++++++
                           
                           ! Pools that are defined in the same way for trees and grasses      
                           bm_sapl(ivm,icir,ifruit,icarbon) = zero
                           bm_sapl(ivm,icir,ilabile,icarbon) = labile_to_total * (bm_sapl(ivm,icir,ileaf,icarbon) + &
                                fcn_root(ivm) * bm_sapl(ivm,icir,iroot,icarbon) + fcn_wood(ivm) * &
                                (bm_sapl(ivm,icir,isapabove,icarbon) + bm_sapl(ivm,icir,isapbelow,icarbon) + &
                                bm_sapl(ivm,icir,icarbres,icarbon)))
                           
                           ! Avoid deciduous PFTs to have leaves out at establishment
                           ! Whether the saplings have leaves or don't have leaves the first year doesn't really matter
                           ! Either the approach is correct in the northern hemisphere or in the southern hemisphere. Note
                           ! that the resource limitation approach starts with the sapling having leaves.
                           ! Anyhow, a spin-up is needed to avoid issues with the initial conditions
                           IF ( pheno_type(ivm) .NE. 1 ) THEN
                              
                              ! Not evergreen. Deciduous PFTs now need to survive an extra year before bud burst. To 
                              ! ensure survival there are several options: (a) either the height of the initial 
                              ! vegetation is increased (this results in more reserves) or (b) the reserves could be
                              ! increased. The second option may result in numerical issues further down
                              ! the code as the optimal reserve level is calculated from the other biomass pools.
                              ! Also some initial tests showed that higher results simply resulted in more respiration.
                              ! Use taller trees to start with.
                              bm_sapl(ivm,icir,icarbres,icarbon) = (bm_sapl(ivm,icir,icarbres,icarbon) + &
                                   bm_sapl(ivm,icir,ileaf,icarbon) + bm_sapl(ivm,icir,iroot,icarbon))
                              bm_sapl(ivm,icir,ileaf,icarbon) = zero
                              bm_sapl(ivm,icir,iroot,icarbon) = zero

                              ! When deciduous trees have no leaves they are senescent
                              senescence(ipts,ivm) = .TRUE.
                              
                           ELSE
                              
                              ! Initilize carbohydrate reserves for evergreen PFTs
                              bm_sapl(ivm,icir,icarbres,icarbon) = zero

                              ! Evergreen trees never go into senescence
                              senescence(ipts,ivm) = .FALSE.
                              
                           ENDIF
                           
                           IF (ld_presc) THEN
                              WRITE(numout,*) '       sapling biomass (gC):',icir,ivm,ipts
                              WRITE(numout,*) '         leaves: (::bm_sapl(ivm,icir,ileaf,icarbon))',&
                                   bm_sapl(ivm,icir,ileaf,icarbon)
                              WRITE(numout,*) '         sap above ground: (::bm_sapl(ivm,icir,ispabove,icarbon)):',&
                                   bm_sapl(ivm,icir,isapabove,icarbon)
                              WRITE(numout,*) '         sap below ground: (::bm_sapl(ivm,icir,isapbelow,icarbon))',&
                                   bm_sapl(ivm,icir,isapbelow,icarbon)
                              WRITE(numout,*) '         heartwood above ground: (::bm_sapl(ivm,icir,iheartabove,icarbon))',&
                                   bm_sapl(ivm,icir,iheartabove,icarbon)
                              WRITE(numout,*) '         heartwood below ground: (::bm_sapl(ivm,icir,iheartbelow,icarbon))',&
                                   bm_sapl(ivm,icir,iheartbelow,icarbon)
                              WRITE(numout,*) '         roots: (::bm_sapl(ivm,icir,iroot,icarbon))',&
                                   bm_sapl(ivm,icir,iroot,icarbon)
                              WRITE(numout,*) '         fruits: (::bm_sapl(ivm,icir,ifruit,icarbon))',&
                                   bm_sapl(ivm,icir,ifruit,icarbon)
                              WRITE(numout,*) '         carbohydrate reserve: (::bm_sapl(ivm,icir,icarbres,icarbon))',&
                                   bm_sapl(ivm,icir,icarbres,icarbon)
                              WRITE(numout,*) '         labile reserve: (::bm_sapl(ivm,icir,ilabile,icarbon))',&
                                   bm_sapl(ivm,icir,ilabile,icarbon)
                           ENDIF
                           
                        ENDDO
                        
                        IF (ld_presc) THEN
                           WRITE(numout,*)'Initial distribution, method 2',ivm
                           WRITE(numout,*)'Average trees height (m): ',ave_tree_height(:)
                           WRITE(numout,*) 'lambda',lambda,'nmaxtrees(ivm)',nmaxtrees(ivm)
                        ENDIF
                        
                        !! 2.7 Determine the biomass in each circumference class.
                        !  I do this based on the biomass in each sapling and the number of trees in each
                        !  circumference class...we need the biomass in an average tree
                        DO icir=1,ncirc
                           
                           circ_class_biomass(ipts,ivm,icir,:,icarbon) = bm_sapl(ivm,icir,:,icarbon)
                           
                        ENDDO

                        ! Now the total biomass is just a sum over all of these
                        DO ipar = 1,nparts
                           
                           biomass(ipts,ivm,ipar,icarbon)= &
                                SUM(circ_class_biomass(ipts,ivm,:,ipar,icarbon)*circ_class_n(ipts,ivm,:))

                        ENDDO

                        ! The light stress should be calculated making use of Pgap so it accounts for LAI
                        ! crown dimensions and tree distribution. However, this would be computationally 
                        ! expensive so we just use a first order estimate based on light attenuation model 
                        ! by Lambert-Beer. When LAI is low, a lot of light reaches the forest floor and so
                        ! KF should increase to make use of the available light by growing leaves
                        lstress_fac = exp(-biomass(ipts,ivm,ileaf,icarbon) * sla(ivm) * 0.5)
                        delta_KF = ABS (KF(ipts,ivm) - ((k_latosa_adapt(ipts,ivm) + &
                          (lstress_fac(ipts,ivm) * (k_latosa_max(ivm)-k_latosa_min(ivm))))) / &
                          ( sla(ivm) * tree_ff(ivm) * pipe_density(ivm) ))
                        KF(ipts,ivm) = ((k_latosa_adapt(ipts,ivm) + &
                          (lstress_fac(ipts,ivm) * &
                          (k_latosa_max(ivm)-k_latosa_min(ivm)))) / &
                          ( sla(ivm) * tree_ff(ivm) * pipe_density(ivm) ))

                        IF(ld_presc)THEN
                           WRITE(numout,*) 'prescribe delta_KF, ', delta_KF
                           WRITE(numout,*) 'prescribe lstress, ', lstress_fac
                        ENDIF
                     END DO

                     IF (ld_presc) THEN
                        WRITE(numout,*)'Initial biomass distribution'
                        DO icir=1,ncirc
                           WRITE(numout,*) circ_class_biomass(ipts,ivm,icir,:,icarbon),circ_class_n(ipts,ivm,icir)
                        ENDDO
                        WRITE(numout,*)'End initial biomass distribution',SUM(circ_class_n(ipts,ivm,:))
                        WRITE(numout,*)"biomass(ipts,ivm,:,icarbon)",biomass(ipts,ivm,:,icarbon)
                     END IF

                     IF (ivm .EQ. test_pft .AND. ld_presc) THEN
                        WRITE(numout,*) 'Check prescribe'
                        WRITE(numout,*) 'stomate_prescribe::init circ_class_n  ',&
                             ipts,ivm,circ_class_n(ipts,ivm,:)
                        DO ipar = 1,nparts
                           WRITE(numout,*) 'biomass vs circ_class_biomass (gC m-2), ',&
                                ipts,ivm,ipar,biomass(ipts,ivm,ipar,icarbon), &
                                SUM(circ_class_biomass(ipts,ivm,:,ipar,icarbon)*circ_class_n(ipts,ivm,:))
                           WRITE(numout,*) 'stomate_prescribe::init biomass  ',&
                                ipts,ivm,circ_class_biomass(ipts,ivm,:,ipar,icarbon)
                        ENDDO
                     ENDIF

                  ! PFT is not present
                  ELSE

                     ! At the tree level
                     circ_class_n(ipts,ivm,:) = zero
                     circ_class_biomass(ipts,ivm,:,:,icarbon) = zero

                     ! At the stand level
                     biomass(ipts,ivm,:,icarbon) = zero
                     ind(ipts,ivm) = zero

                  ENDIF

                  ! Set leaf age classes, all leaves are current year leaves
                  leaf_frac(ipts,ivm,:) = zero
                  leaf_frac(ipts,ivm,1) = un

                  !+++CHECK+++
                  ! Set time since last beginning of growing season but only
                  ! for the first day of the whole simulation. When the model
                  ! is initialized when_growthinit is set to undef. In subsequent
                  ! time steps it should have a value. For trees without phenology
                  ! the growing season starts at the moment the PFT is prescribed
                  IF (when_growthinit(ipts,ivm) .EQ. undef) THEN
                     when_growthinit(ipts,ivm) = 200
                  ENDIF
                  !+++++++++++

                  ! Seasonal trees have no leaves at beginning
                  ! Saplings of evergreen trees have a leaf mass on day 1 and mass in the other components
                  ! saplings of deciduous trees have no leaves on day 1 but mass in the other components.
                  IF ( pheno_model(ivm) .NE. 'none' ) THEN

                     ! Add the carbon from the leaves to the reserve pool
                     biomass(ipts,ivm,icarbres,icarbon) = biomass(ipts,ivm,icarbres,icarbon) + biomass(ipts,ivm,ileaf,icarbon)
                     biomass(ipts,ivm,ileaf,icarbon) = zero
                     leaf_frac(ipts,ivm,1) = zero

                     !+++CHECK+++
                     ! Set time since last beginning of growing season but only
                     ! for the first day of the whole simulation. When the model
                     ! is initialized when_growthinit is set to undef. In subsequent
                     ! time steps it should have a value. The phenology module 
                     ! prevents leaf onset soon after senescence, by setting 
                     ! ::when_growthinit to a value, leaf offset 
                     ! will occur at the first opportunity 
!!$                     when_growthinit(ipts,ivm) = large_value
                     IF (when_growthinit(ipts,ivm) .EQ. undef) THEN
                        when_growthinit(ipts,ivm) = 200
                     ENDIF
                     !++++++++++++

                     ! Redundant, flag has already been set. When there are no leaves, the tree is in senescence
                     senescence(ipts,ivm) = .TRUE.

                  ENDIF ! pheno_model(ivm)

                  ! The biomass to build the saplings is taken from the atmosphere, keep track of
                  ! amount to calculate the C-balance closure
                  co2_to_bm(ipts,ivm) = co2_to_bm(ipts,ivm) + ( SUM(biomass(ipts,ivm,:,icarbon))  / dt )          

               ENDIF ! tree(ivm)

               !! 2.8 Initialize grassy PFTs
               !! Use veget_max to check whether the PFT is present. If the PFT is present but it 
               !! has no biomass, prescribe its biomass (gC m-{2}). It is assumed that at day 1 
               !! grasses have all their biomass in the reserve pool. The criteria exclude crops.
               !! Crops are no longer prescribed but planted the day that begin_leaves is true.    
               
               !+++TEMP+++
               IF(ld_presc .AND. test_pft == ivm)THEN
                  WRITE(numout,*) 'prescribe - total biomass, ',SUM(biomass(ipts,ivm,:,icarbon))
               ENDIF
               !++++++++++

               IF ( ( .NOT. is_tree(ivm) ) .AND. &
                    ( natural(ivm) ) .AND. &
                    ( veget_max(ipts,ivm) .GT. min_stomate ) .AND. &
                    ( SUM( biomass(ipts,ivm,:,icarbon) ) .LE. min_stomate ) ) THEN

                  !+++TEMP+++
                  IF(ld_presc .AND. test_pft == ivm)THEN
                     WRITE(numout,*) 'We will prescribe a new vegetation, the old one died'
                  ENDIF
                  !++++++++++

                  ! For grasses we assume that an individual grass is 1 m2 of grass. This is set
                  ! in nmaxtrees in pft_parameters.f90. It could be set to another value but 
                  ! with the current code this should not have any meaning. The grassland
                  ! does not necessarily covers the whole 1 m2 so adjust for the canopy cover
                  ind(ipts,ivm) = nmaxtrees(ivm) * ha_to_m2 * canopy_cover(ivm)

                  ! now we generate the size of a single grass sapling...this was all taken from
                  ! stomate_data.f90...we do not deal with circumference classes for grasses
                  ! and crops, but we want to keep the arrays the same as for the trees so
                  ! we put the sapling information into the first circumference class
                
                  !+++CHECK+++
                  ! Calculate the sapwood to leaf mass in a similar way as has been done for trees.
                  ! For trees this approach had been justified by observations. For grasses such
                  ! justification is not supported by observations but we didn't try to find it.
                  ! Needs more work by someone interested in grasses. There might be a more elegant
                  ! solution making use of a well observed parameter. 
!!$                  k_latosa(ipts,ivm) = k_latosa_min(ivm) + &
!!$                       (wstress_fac(ipts,ivm) * lstress_fac(ipts,ivm) * &
!!$                       (k_latosa_max(ivm)-k_latosa_min(ivm)))
!!$                  k_latosa(ipts,ivm) = wstress_fac(ipts,ivm) * (k_latosa_min(ivm) + &
!!$                       (lstress_fac(ipts,ivm) * &
!!$                       (k_latosa_max(ivm)-k_latosa_min(ivm))))
                  k_latosa(ipts,ivm) = (k_latosa_adapt(ipts,ivm) + &
                       (lstress_fac(ipts,ivm) * &
                       (k_latosa_max(ivm)-k_latosa_min(ivm))))

                  ! The mass of the structural carbon relates to the mass of the leaves through
                  ! a prescribed parameter ::k_latosa
                  KF(ipts,ivm) = k_latosa(ipts,ivm)
                  !+++++++++++

                  ! Calculate leaf to root area    
                  LF(ipts,ivm) = c0_alloc(ipts,ivm) * KF(ipts,ivm)

                  !---TEMP---
                  IF(ld_presc .AND. test_pft == ivm)THEN
                     WRITE(numout,*) 'KF, ', k_latosa(ipts,ivm), KF(ipts,ivm)
                     WRITE(numout,*) 'LF, c0_alloc, ', c0_alloc(ipts,ivm) * KF(ipts,ivm), &
                       c0_alloc(ipts,ivm)
                  ENDIF
                  !----------

                  ! initialize everything to make sure there are not random values floating around
                  ! for ncirc != 1
                  bm_sapl(ivm,:,:,icarbon) = val_exp

                  ! Similar as for trees, the initial height of the vegetation was defined 
                  bm_sapl(ivm,1,ileaf,icarbon) = height_init_min(ivm) / lai_to_height(ivm) / sla(ivm)

                  ! Use allometric relationships to define the root mass based on leaf mass. Some 
                  ! sapwood mass is needed to store the reserves. An arbitrairy fraction of 5% was
                  ! used 
                  bm_sapl(ivm,1,iroot,icarbon) = bm_sapl(ivm,1,ileaf,icarbon) / LF(ipts,ivm)
                  bm_sapl(ivm,1,isapabove,icarbon) = bm_sapl(ivm,1,ileaf,icarbon) / KF(ipts,ivm)

                  ! Some of the biomass components that exist for trees are undefined for grasses
                  bm_sapl(ivm,1,isapbelow,icarbon) = zero
                  bm_sapl(ivm,1,iheartabove,icarbon) = zero
                  bm_sapl(ivm,1,iheartbelow,icarbon) = zero
                  bm_sapl(ivm,1,ifruit,icarbon) = zero
                  bm_sapl(ivm,1,icarbres,icarbon) = zero

                  ! Pools that are defined in the same way for trees and grasses      
                  bm_sapl(ivm,1,ilabile,icarbon) = labile_to_total * (bm_sapl(ivm,1,ileaf,icarbon) + &
                       fcn_root(ivm) * bm_sapl(ivm,1,iroot,icarbon) + fcn_wood(ivm) * &
                       (bm_sapl(ivm,1,isapabove,icarbon) + bm_sapl(ivm,1,isapbelow,icarbon) + &
                       bm_sapl(ivm,1,icarbres,icarbon)))

                  ! Avoid deciduous PFTs to have leaves out at establishment
                  ! Whether the saplings have leaves or don't have leaves the first year doesn't really matter
                  ! Either the approach is correct in the northern hemisphere or in the southern hemisphere. Note
                  ! that the resource limitation approach starts with the sapling having leaves.
                  ! Anyhow, a spin-up is needed to avoid issues with the initial conditions
                  IF ( pheno_type(ivm) .NE. 1 ) THEN

                     ! Not evergreen. Deciduous PFTs now need to survive an extra year before bud burst. To 
                     ! ensure survival there are several options: (a) either the height of the initial 
                     ! vegetation is increased (this results in more reserves) or (b) the reserves could be
                     ! increased. The second option may result in result in numerical issues further down
                     ! the code as the optimal reserve level is calculated from the other biomass pools
                     bm_sapl(ivm,1,icarbres,icarbon) = bm_sapl(ivm,1,icarbres,icarbon) + bm_sapl(ivm,1,ileaf,icarbon) + &
                          bm_sapl(ivm,1,iroot,icarbon)
                     bm_sapl(ivm,1,ileaf,icarbon) = zero
                     bm_sapl(ivm,1,iroot,icarbon) = zero

                     ! When there are no leaves, the crop/grass is in senescence
                     senescence(ipts,ivm) = .TRUE.

                  ELSE

                     ! Initilize carbohydrate reserves for evergreen PFTs
                     bm_sapl(ivm,1,icarbres,icarbon) = zero 

                     ! Evergreen plants never go into senescence
                     senescence(ipts,ivm) = .FALSE.

                  ENDIF

                  IF (ld_presc) THEN
                     WRITE(numout,*) '       sapling biomass (gC):',1,ivm,ipts
                     WRITE(numout,*) '         leaves: (::bm_sapl(ivm,1,ileaf,icarbon))',&
                          bm_sapl(ivm,1,ileaf,icarbon)
                     WRITE(numout,*) '         sap above ground: (::bm_sapl(ivm,1,ispabove,icarbon)):',&
                          bm_sapl(ivm,1,isapabove,icarbon)
                     WRITE(numout,*) '         sap below ground: (::bm_sapl(ivm,1,isapbelow,icarbon))',&
                          bm_sapl(ivm,1,isapbelow,icarbon)
                     WRITE(numout,*) '         heartwood above ground: (::bm_sapl(ivm,1,iheartabove,icarbon))',&
                          bm_sapl(ivm,1,iheartabove,icarbon)
                     WRITE(numout,*) '         heartwood below ground: (::bm_sapl(ivm,1,iheartbelow,icarbon))',&
                          bm_sapl(ivm,1,iheartbelow,icarbon)
                     WRITE(numout,*) '         roots: (::bm_sapl(ivm,1,iroot,icarbon))',&
                          bm_sapl(ivm,1,iroot,icarbon)
                     WRITE(numout,*) '         fruits: (::bm_sapl(ivm,1,ifruit,icarbon))',&
                          bm_sapl(ivm,1,ifruit,icarbon)
                     WRITE(numout,*) '         carbohydrate reserve: (::bm_sapl(ivm,1,icarbres,icarbon))',&
                          bm_sapl(ivm,1,icarbres,icarbon)
                     WRITE(numout,*) '         labile reserve: (::bm_sapl(ivm,1,ilabile,icarbon))',&
                          bm_sapl(ivm,1,ilabile,icarbon)
                  ENDIF

                  ! Write initial values
                  IF (ld_presc) THEN
                     WRITE(numout,*) '       root to sapwood tradeoff (LF) : ', c0_alloc(ipts,ivm)
                     WRITE(numout,*) '       grass sapling biomass: ',bm_sapl(ivm,1,:,icarbon)
                  ENDIF

                  ! Initial biomass (g C m-2)
                  biomass(ipts,ivm,:,icarbon) = bm_sapl(ivm,1,:,icarbon) * ind(ipts,ivm)
                  IF (ld_presc) THEN 
                     WRITE(numout,*) 'bm_grass, prescribe, ', biomass(ipts,ivm,:,icarbon)
                  ENDIF

                  ! Synchronize biomass and circ_class_biomass (gC tree-1). This 
                  ! allows to more easily check the mass balance closure
                  circ_class_biomass(ipts,ivm,1,:,icarbon) = bm_sapl(ivm,1,:,icarbon)
                  circ_class_n(ipts,ivm,1) = ind(ipts,ivm)

                  ! Set leaf age classes -> all leaves will be current year leaves
                  leaf_frac(ipts,ivm,:) = zero
                  leaf_frac(ipts,ivm,1) = un

                  ! Set time since last beginning of growing season but only
                  ! for the first day of the whole simulation. When the model
                  ! is initialized when_growthinit is set to undef. In subsequent
                  ! time steps it should have a value.
!!$                  when_growthinit(ipts,ivm) = large_value
                  IF (when_growthinit(ipts,ivm) .EQ. undef) THEN
                     when_growthinit(ipts,ivm) = 200
                  ENDIF

                  ! The biomass to build the saplings is taken from the atmosphere, keep track of
                  ! amount to calculate the C-balance closure
                  co2_to_bm(ipts,ivm) = co2_to_bm(ipts,ivm) + ( SUM(biomass(ipts,ivm,:,icarbon))  / dt )

               ELSEIF (.NOT. natural(ivm)) THEN

                  ! Initialize croplands - else leaves_begin will never become true
                  ! Set time since last beginning of growing season but only
                  ! for the first day of the whole simulation. When the model
                  ! is initialized when_growthinit is set to undef. In subsequent
                  ! time steps it should have a value.
                  IF (when_growthinit(ipts,ivm) .EQ. undef) THEN
                     when_growthinit(ipts,ivm) = 200
                  ENDIF

               ENDIF ! .NOT. tree(ivm)

               !! 2.3 Declare PFT present 
               !! Now that the PFT has biomass it should be declared 'present'
               !! everywhere in that grid box. Assign some additional properties
               PFTpresent(ipts,ivm) = .TRUE.
               everywhere(ipts,ivm) = un
               age(ipts,ivm) = zero
               npp_longterm(ipts,ivm) = npp_longterm_init
               lm_lastyearmax(ipts,ivm) = zero

            ENDIF   ! not control%ok_dgvm  or agricultural

         ENDIF ! IF (veget_max .GT. zero .AND. ind .EQ. zero)

      ENDDO ! loop over pixels

   ENDDO ! loop over PFTs

 !! 3. Scynchronize biomass and circ_class_biomass 
   
   ! For trees both biomass and circ_class_biomass are recorded. Where biomass
   ! contains the stand level biomass of the different components (gC m-2), 
   ! circ_class_biomass contains the biomass of a model tree in each circumference
   ! class (gC tree-1). When multiplied with the number of trees in each circumference
   ! class (::circ_class_n), biomass and circ_class_biomass should be identical.
   ! This has been checked in all routines where biomass is calculated and at the
   ! daily level consistency between both variables is maintained. However, due
   ! to precision issues and the fact tha biomass and circ_class_biomass are 
   ! cumulative variables, the difference between both variables accumulates to 
   ! 0.00001 at the annual level. To avoid propagation of the precision error
   ! both variables are synchronized daily.

   ! Initialze the variables for the synchronisation
   sync_ind(:,:) = zero
   sync_biomass(:,:,:,:) = zero

   ! Synchronize
   DO ivm = 1,nvm

      IF (is_tree(ivm)) THEN

         DO icir = 1,ncirc

            ! Temporary stand density 
            sync_ind(:,ivm) = sync_ind(:,ivm) + circ_class_n(:,ivm,icir)

            DO iele = 1,nelements

               ! Temporary biomass
               DO ipar = 1,nparts

                  sync_biomass(:,ivm,ipar,iele) = sync_biomass(:,ivm,ipar,iele) + &
                       circ_class_n(:,ivm,icir) * circ_class_biomass(:,ivm,icir,ipar,iele)

               ENDDO ! nparts

            ENDDO ! nelements

         ENDDO ! ncirc

         ! Synchronize stand density 
         DO ipts = 1,npts

            DO iele = 1,nelements

               IF (ivm .EQ. test_pft .AND. ABS(sync_ind(ipts,ivm) - ind(ipts,ivm)) .GT. min_stomate ) THEN

                  IF (ld_presc) THEN
                     WRITE(numout,*) 'Divergence of density between ::ind and ::circ_class_n'
                     WRITE(numout,*) 'This problem has been identified in stomate_prescribe but could'
                     WRITE(numout,*) 'be caused in any routine calculating stand density'
                  ENDIF

               ELSE

                  ind(ipts,ivm) = sync_ind(ipts,ivm)

               ENDIF

               ! Synchronize biomass
               IF (ld_presc .AND. ivm .EQ. test_pft)THEN
                  IF( SUM(biomass(ipts,ivm,:,iele)) .NE. zero)THEN
                     IF( SUM( ABS(sync_biomass(ipts,ivm,:,iele) - biomass(ipts,ivm,:,iele) ) ) / &
                          SUM(biomass(ipts,ivm,:,iele)) .GT. 0.0001 ) THEN

                        WRITE(numout,*) 'Divergence of biomass between ::biomass and ::circ_class_biomass'
                        WRITE(numout,*) 'This problem has been identified in stomate_prescribe but could'
                        WRITE(numout,*) 'caused in any routine calculating biomass components'
                        WRITE(numout,*) '% divergence', ivm, &
                             SUM( ABS(sync_biomass(ipts,ivm,:,iele) - biomass(ipts,ivm,:,iele) ) ) / &
                             SUM(biomass(ipts,ivm,:,iele)), &
                             SUM(sync_biomass(ipts,ivm,:,iele)), SUM(biomass(ipts,ivm,:,iele))
                     ENDIF
                  ENDIF
               ENDIF

               biomass(ipts,ivm,:,iele) = sync_biomass(ipts,ivm,:,iele)


            ENDDO ! nelements

         ENDDO ! npts

      ENDIF ! is_tree

   ENDDO ! # PFT's

 !! 4. Calculate components of the mass balance

   !! 4.1 Calculate final biomass
   pool_end(:,:,:) = zero 
   DO ipar = 1,nparts
      DO iele = 1,nelements
         pool_end(:,:,iele) = pool_end(:,:,iele) + &
              (biomass(:,:,ipar,iele) * veget_max(:,:))
      ENDDO
   ENDDO

   !! 4.2 Calculate mass balance
   check_intern(:,:,iatm2land,icarbon) = check_intern(:,:,iatm2land,icarbon) + &
        co2_to_bm(:,:) * veget_max(:,:) * dt
   check_intern(:,:,iland2atm,icarbon) = -un * zero
   check_intern(:,:,ilat2out,icarbon) = zero
   check_intern(:,:,ilat2in,icarbon) = -un * zero
   check_intern(:,:,ipoolchange,icarbon) = -un * (pool_end(:,:,icarbon) - pool_start(:,:,icarbon))
   closure_intern = zero
   DO imbc = 1,nmbcomp
      closure_intern(:,:,icarbon) = closure_intern(:,:,icarbon) + check_intern(:,:,imbc,icarbon)
   ENDDO

   !! 4.3 Write outcome
   DO ipts=1,npts
      DO ivm=1,nvm
         IF(ABS(closure_intern(ipts,ivm,icarbon)) .LE. min_stomate)THEN
            IF (ld_massbal) WRITE(numout,*) 'Mass balance closure in prescribe_prognostic'
         ELSE
            WRITE(numout,*) 'Error: mass balance is not closed in prescribe_prognostic'
            WRITE(numout,*) '   ipts,ivm; ', ipts,ivm
            WRITE(numout,*) '   Difference is, ', closure_intern(ipts,ivm,icarbon)
            WRITE(numout,*) '   pool_end,pool_start: ', pool_end(ipts,ivm,icarbon), pool_start(ipts,ivm,icarbon)
            WRITE(numout,*) '   check_intern,co2_to_bm,pool_end,veget_max: ', &
                 check_intern(ipts,ivm,iatm2land,icarbon),co2_to_bm(ipts,ivm), veget_max(ipts,ivm)
            IF(ld_stop)THEN
               CALL ipslerr_p (3,'prescribe_prognostic', 'Mass balance error.','','')
            ENDIF
         ENDIF
      ENDDO
   ENDDO
   

   END SUBROUTINE prescribe_prognostic

END MODULE stomate_prescribe
